/*
 *    Copyright (C) 2002 Sergej Bogomolov <bog@deol.ru>
 *    This filter extract color plane from YUV4MPEG2
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */
#include <config.h>
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <yuv4mpeg.h>
#include <mjpeg_logging.h>

int verbose = 1;
uint8_t	*input_frame[3];
uint8_t	*output_frame[3];

void	filter(int width, int height, uint8_t *input[], uint8_t * const output[]);

static void Usage(char *name )
{
	fprintf(stderr,
		"Usage: %s: [-h] [-p plane] [-v num]\n"
                        "-h   - Print out this help\n"
			"-p   - plane (Y, U, V)\n"
			"-v   - Verbosity [0..2]\n", name);
}
			
int
main(int argc, char *argv[])
{
	int	input_fd = 0, output_fd = 1;
	int	horz, vert;
	int	c, i, plane = 1;
	int	frame_count;

	y4m_stream_info_t istream, ostream;
	y4m_frame_info_t iframe;

	while((c = getopt(argc, argv, "p:v:h")) != EOF) {
		switch(c) {
		    case 'p':
				if( strcmp( optarg, "Y" ) == 0 ) plane = 1;
				if( strcmp( optarg, "U" ) == 0 ) plane = 2;
				if( strcmp( optarg, "V" ) == 0 ) plane = 3;
				break;
		    case 'v':
			verbose = atoi (optarg);
			if( verbose < 0 || verbose >2 )
			{
				Usage (argv[0]);
				exit (1);
			}
			break;		  
			
		case 'h':
                        Usage (argv[0]);
		default:
			exit(0);
		}
	}

	(void)mjpeg_default_handler_verbosity(verbose);

	y4m_init_stream_info(&istream);
	y4m_init_stream_info(&ostream);
	y4m_init_frame_info(&iframe);

	i = y4m_read_stream_header(input_fd, &istream);
	if (i != Y4M_OK)
		mjpeg_error_exit1("Input stream error: %s", y4m_strerr(i));

	if( istream.height % 2 != 0 )
	{
		mjpeg_error_exit1("Input images have odd number of lines" );
	}
	if( istream.width % 2 != 0 )
	{
		mjpeg_error_exit1("Input images have odd number of pixels" );
	}
	horz = istream.width;
	vert = istream.height;
	mjpeg_debug( "width=%d height=%d plane=%d", horz, vert, plane);

	y4m_copy_stream_info(&ostream, &istream);

	input_frame[0] = malloc(horz * vert);
	input_frame[1] = malloc((horz / 2) * (vert / 2));
	input_frame[2] = malloc((horz / 2) * (vert / 2));

	output_frame[0] = malloc(horz * vert);
	output_frame[1] = malloc((horz / 2) * (vert / 2));
	output_frame[2] = malloc((horz / 2) * (vert / 2));

/*	y4m_write_stream_header(output_fd, &ostream); */

	frame_count = 0;
	while (y4m_read_frame(input_fd, &istream, &iframe, input_frame) == Y4M_OK)
	{ 		
	  filter(horz, vert,  input_frame, output_frame);
/*	  y4m_write_frame(output_fd, &ostream, &iframe, output_frame); */
	  if( plane == 1 ) write( output_fd, output_frame[0], horz * vert );
	  if( plane == 2 ) write( output_fd, output_frame[1], horz * vert / 4);
	  if( plane == 3 ) write( output_fd, output_frame[2], horz * vert / 4);
	  mjpeg_debug( "frame=%d", frame_count );
	  frame_count++;
	}

	y4m_fini_stream_info(&istream);
/*	y4m_fini_stream_info(&ostream); */
	y4m_fini_frame_info(&iframe);
	exit(0);
}

void
filter(int width, int height, uint8_t *input[], uint8_t * const output[])
{
	memcpy( output[0], input[0], width*height );
	memcpy( output[1], input[1], width*height/4 );
	memcpy( output[2], input[2], width*height/4 );
}
